SystemOrganization addCategory: #'Idiom-Exercise'!

TestCase subclass: #IdiomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:08'!
evaluate: block andAssertTakesLessThan: time
	"returns the time it took to evaluate the passed method"

	| millisecondsBeforeRunning millisecondsAfterRunning |
	
	millisecondsBeforeRunning := Time millisecondClockValue.
	block value.
	millisecondsAfterRunning := Time millisecondClockValue.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < time
	! !

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:52'!
expectErrorOn: block withError: errorClass andAssert: anotherBlock
	"evaluates a block expecting an error, and fails if it doesn't throw an exception. If it does, evaluates the second block passing the error as parameter"
	
	[block value.
	self fail.]
		on: errorClass 
		do: anotherBlock 
! !

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:10'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook|
	
	customerBook := CustomerBook  new.
	
	self evaluate: [customerBook addCustomerNamed: 'John Lennon'] andAssertTakesLessThan: 50.
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:10'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	  
	self evaluate:  [customerBook removeCustomerNamed: johnLennon] andAssertTakesLessThan: 100.
! !

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:34'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	self expectErrorOn: [ customerBook addCustomerNamed: ''] withError: Error  andAssert: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ].
! !

!IdiomTest methodsFor: 'testing' stamp: 'NicolasRavasi 5/9/2012 20:36'!
test04CanNotAddRepeatedCustomer 
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	self expectErrorOn: [customerBook removeCustomerNamed: 'Paul McCartney'] withError: NotFound  andAssert: [ :anError |
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon)].

! !

Object subclass: #CustomerBook
	instanceVariableNames: 'customers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:57'!
customerAlreadyExistsErrorMessage

	^'Customer already exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:53'!
customerCanNotBeEmptyErrorMessage

	^'Customer name cannot be empty'! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:42'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(customers includes: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	customers add: aName ! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:56'!
includesCustomerNamed: aName

	^customers includes: aName ! !

!CustomerBook methodsFor: 'initialization' stamp: 'HernanWilkinson 7/6/2011 17:42'!
initialize

	super initialize.
	customers := OrderedCollection new! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:48'!
isEmpty
	
	^customers isEmpty  ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:56'!
numberOfCustomers
	
	^customers size! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:44'!
removeCustomerNamed: aName
 
	customers remove: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !
